#include <stdio.h>
#include <stdlib.h>
#include "ibitmap.h"

struct palette {
	int r, g, b, a;
};

typedef struct palette PALETTE;

static int get_next_4_bytes(FILE *f)
{
	int one = (fgetc(f) & 0xFF);
	int two = (fgetc(f) & 0xFF) << 8;
	int three = (fgetc(f) & 0xFF) << 16;
	int four = (fgetc(f) & 0xFF) << 24;
	return one | two | three | four;
}

static int get_next_2_bytes(FILE *f)
{
	int one = (fgetc(f) & 0xFF);
	int two = (fgetc(f) & 0xFF) << 8;
	return one | two;
}

static int get_next_byte(FILE *f)
{
	return fgetc(f) & 0xFF;
}

void load_bitmap(IBITMAP *bm, const char *filename)
{
	FILE *f = fopen(filename, "r");
	int color_depth, n_colors, dummy;
	
	if (!f) {
		fprintf(stderr, "could not open %s.\n", filename);
		exit(1);
	}
	if (get_next_byte(f) != 'B' || get_next_byte(f) != 'M') {
		fprintf(stderr, "%s does not seem to be a proper bitmap file.\n", filename);
		exit(1);
	}
	fprintf(stderr, ".");
	dummy = get_next_4_bytes(f);
	fprintf(stderr, ".");
	dummy = get_next_4_bytes(f);
	fprintf(stderr, ".");
	dummy = get_next_4_bytes(f);
	fprintf(stderr, ".");
	dummy = get_next_4_bytes(f);
	fprintf(stderr, ".");
	bm->width = get_next_4_bytes(f);
	bm->height = get_next_4_bytes(f);
	fprintf(stderr, "width: %d, height: %d\n", bm->width, bm->height);
	dummy = get_next_2_bytes(f);
	color_depth = get_next_2_bytes(f);
	dummy = get_next_4_bytes(f);
	dummy = get_next_4_bytes(f);
	dummy = get_next_4_bytes(f);
	dummy = get_next_4_bytes(f);
	n_colors = get_next_4_bytes(f);
	dummy = get_next_4_bytes(f);
	
	bm->pixels = malloc(sizeof(struct pixel) * bm->width * bm->height);
	
	if (color_depth < 16) {
		int palette_size = 1 << color_depth;
		PALETTE *pal = malloc(sizeof(PALETTE) * palette_size);
		int i, x, y;
		
		for (i = 0; i < palette_size; ++i) {
			pal[i].b = get_next_byte(f);
			pal[i].g = get_next_byte(f);
			pal[i].r = get_next_byte(f);
			pal[i].a = get_next_byte(f);
		}
		for (y = 0; y < bm->height; ++y) {
			for (x = 0; x < bm->width; ++x) {
				int c = get_next_byte(f);
				PALETTE p = pal[c];
				bm->pixels[x + y * bm->width].b = (unsigned char)p.b;
				bm->pixels[x + y * bm->width].g = (unsigned char)p.g;
				bm->pixels[x + y * bm->width].r = (unsigned char)p.r;
			}
		}
	}
	else {
		int x, y;
		
		for (y = 0; y < bm->height; ++y) {
			for (x = 0; x < bm->width; ++x) {
				bm->pixels[x +y * bm->width].b = (unsigned char)get_next_byte(f);
				bm->pixels[x +y * bm->width].g = (unsigned char)get_next_byte(f);
				bm->pixels[x +y * bm->width].r = (unsigned char)get_next_byte(f);
			}
		}
	}
	fclose(f);
}